using Syntriax.Engine.Core.Abstract;
using Syntriax.Engine.Core.Exceptions;

namespace Syntriax.Engine.Core.Factory;

public class BehaviourFactory
{
    public static T Instantiate<T>(IHierarchyObject hierarchyObject, params object?[]? args) where T : class, IBehaviour
        => Instantiate<T>(hierarchyObject, stateEnable: null, args);

    public static T Instantiate<T>(IHierarchyObject hierarchyObject, IStateEnable? stateEnable, params object?[]? args)
        where T : class, IBehaviour
    {
        T behaviour = TypeFactory.Get<T>(args);

        stateEnable ??= TypeFactory.Get<StateEnable>();
        if (!stateEnable.Assign(behaviour))
            throw AssignFailedException.From(stateEnable, behaviour);

        if (!behaviour.Assign(stateEnable))
            throw AssignFailedException.From(behaviour, stateEnable);
        if (!behaviour.Assign(hierarchyObject.BehaviourController))
            throw AssignFailedException.From(behaviour, hierarchyObject.BehaviourController);

        return behaviour;
    }
}
